/**
 * @file libkyaccessctl.h
 * @author lixiuwen (lixiuwen@kylinos.cn)
 * @brief 安全SDK接口门禁管控
 * @version 0.1
 * @date 2024-08-06
 * 
 * @copyright Copyright (c) 2024
 * @defgroup 安全SDK接口门禁管控
 * 
 */
#ifndef LIBKYACCESSCTL_H
#define LIBKYACCESSCTL_H

#ifdef __cplusplus
extern "C"{
#endif


//接口访问权限
typedef enum _kdk_accessctl_status {
   KDK_ACCESSCTL_DISABLE= 0,      //禁止访问
   KDK_ACCESSCTL_ENABLE = 1       //允许访问
}kdk_accessctl_status;


/**
 * @brief 获取访问接口权限（在本函数中记录接口被调用的审计日志、访问接口权限日志）
 * 
 * @param char* module 被调用接口的模块名（软件包为一级模块，软件包名为一级模块名。当软件包各接口访问策略相同时，接口对应一级模块，模块名格式：一级模块名；当软件包各接口访问策略不同时，将一级模块划分为多级模块，接口对应多级模块，模块名格式：一级模块名:多级模块名。）
 * @param format,... 格式化参数，设置自定义信息
 * @return int status 接口访问权限，参见kdk_accessctl_status。
 */
#define kdk_accessctl_call(module, format, ...) \
({\
    int status = KDK_ACCESSCTL_ENABLE;\
    struct kdk_accessctl_caller_item *item = kdk_accessctl_create_item(-1, -1, __FUNCTION__, module, format, ##__VA_ARGS__);\
    status = kdk_accessctl_check_callable(item);\
    kdk_accessctl_release_item(item);\
    status;\
})

/**
 * @brief 获取访问接口权限（在本函数中仅记录访问接口权限日志）
 * 
 * @param char* module 被调用接口的模块名（软件包为一级模块，软件包名为一级模块名。当软件包各接口访问策略相同时，接口对应一级模块，模块名格式：一级模块名；当软件包各接口访问策略不同时，将一级模块划分为多级模块，接口对应多级模块，模块名格式：一级模块名:多级模块名。）
 * @param format,... 格式化参数，设置自定义信息
 * @return int status 接口访问权限，参见kdk_accessctl_status。
 */
#define kdk_accessctl_call_without_inlog(module, format, ...) \
({\
    int status = KDK_ACCESSCTL_ENABLE;\
    struct kdk_accessctl_caller_item *item = kdk_accessctl_create_item(-1, -1, __FUNCTION__, module, format, ##__VA_ARGS__);\
    kdk_accessctl_set_inlog(item, 0);\
    status = kdk_accessctl_check_callable(item);\
    kdk_accessctl_release_item(item);\
    status;\
})


/**
 * @brief 获取指定模块策略
 * 
 * @param module 模块名（软件包为一级模块，软件包名为一级模块名。当模块为一级模块时，模块名格式：一级模块名；当模块为多级模块时，模块名格式：一级模块名:多级模块名。）
 * @return char* 指定模块策略，cjson结构（当指定模块为一级模块时，获取一级模块所有策略；当指定模块为多级模块时，获取对应的多级模块策略；当没有该模块时，返回null。）
 */
char* kdk_accessctl_get_cuspolicy(const char* module);

/**
 * @brief 更新指定模块策略
 * 
 * @param module 模块名（软件包为一级模块，软件包名为一级模块名。当模块为一级模块时，模块名格式：一级模块名；当模块为多级模块时，模块名格式：一级模块名:多级模块名）
 * @param policy 模块策略，cjson结构（当模块为一级模块时，对应一级模块策略；当模块为多级模块时，对应多级模块策略。）

 * @return int 0：失败；1：成功（当模块为一级模块时，更新一级模块所有策略；当模块为多级模块时，更新对应的多级模块策略。当软件包没有注册模块，返回失败。当软件包已注册模块，但策略中没有该模块时，则新增模块。）
 */
int kdk_accessctl_update_cuspolicy(const char* module, const char* policy);



/**
 * @brief 创建门禁管控调用者结构体
 * 
 * @param uid 调用者用户id
 * @param pid 调用者进程id
 * @param func 被调用方法名
 * @param module 模块名
 * @param format,...  格式化参数，设置自定义信息
 * @return struct kdk_accessctl_caller_item* 门禁管控调用者结构体
 */
struct kdk_accessctl_caller_item * kdk_accessctl_create_item(int uid, int pid, const char* func, const char* module, const char* format, ...);

/**
 * @brief 释放门禁管控调用者结构体
 * 
 * @param item 门禁管控调用者结构体
 */
void kdk_accessctl_release_item(struct kdk_accessctl_caller_item *item);

/**
 * @brief 获取访问接口权限
 * 
 * @param item 门禁管控调用者结构体
 * @return int 是否被允许调用，参见kdk_accessctl_status
 */
int kdk_accessctl_check_callable(struct kdk_accessctl_caller_item* item); 

/**
 * @brief 设置是否记录调用门禁日志
 * 
 * @param item 门禁管控调用者结构体
 * @param logSign 调用门禁日志标识。1为记录调用门禁日志；0为不记录调用门禁日志。
 */
void kdk_accessctl_set_inlog(struct kdk_accessctl_caller_item* item, int logSign);

#ifdef __cplusplus
}
#endif

#endif